import java.util.*;

public class CaddyVM
{

/* Registers */
int 	    regA = 0;
float 	    regB = 0.0f;
String 	    regC = "";
Vector      regD;

/* Special Registers */
int         regOpcodePointer = 0;
Vector      regSQL = new Vector(0,1);
Vector      regScriptData = new Vector(0,1);
Hashtable   regHashVars = null;

/* Flags */
boolean     flagZ = true; //Flag Zero

/* SQL support */
CaddySQL sql;// = new CaddySQL();

//Set our debug level
boolean m_bDebug = false;

public CaddyVM(CaddySQL _sql)
{
    //Grab a pointer to our CaddySQL object
    sql = _sql;
}

void addCommand(String command)
{
     regScriptData.add(command);
}

void setHashVars(Hashtable hashArguments)
{
      regHashVars =  hashArguments;
}

String executeScript()
{
    StringBuffer results = new StringBuffer();
    for(; regOpcodePointer < regScriptData.size(); regOpcodePointer++)
    {
         results.append( parseCommand( (String)regScriptData.elementAt(regOpcodePointer) ) );
    }
    //temp.append((String)regScriptData.elementAt(i)+"\n");
    regScriptData.clear();
    return results.toString();
}

String parseCommand(String sCommand)
{
    //split it up.
    StringTokenizer ST = new StringTokenizer(sCommand , " ", true);
    boolean bFirst = true;
    String command = "";
    StringBuffer argument = new StringBuffer();
    while(ST.hasMoreTokens())
    {
      if(bFirst)
      {
        command = ST.nextToken();
        bFirst = false;
      }
      else
      {
        argument.append(ST.nextToken());
      }
    }
    
    // Handle keywords
    if(command.compareTo("SQLQUERY") == 0)
    {
      DebugPrint("/SQLQUERY");
      String convertedSQLQuery = parseArguments(argument.toString());
      if(sql.Query(convertedSQLQuery, regSQL) == 0)
      {
         //Set our A register
         regA = regSQL.size();
      }
      else
      {
        //error
        System.out.println("ERROR: Invalid SQL Query");
      }
    }
    else
    if(command.compareTo("SQLNEXT") == 0)
    {
        DebugPrint("/SQLNEXT");
        regD = (Vector)regSQL.elementAt(regSQL.size() - regA);
        DecA();
    }
    else
    if(command.compareTo("SQLPRINT") == 0)
    {
        DebugPrint("/SQLPRINT");
        StringTokenizer StNewArg = new StringTokenizer(argument.toString(), " ");
        try
        {
          int nElement = Integer.valueOf(StNewArg.nextToken()).intValue();
          return (String)regD.elementAt(nElement);

        }
        catch(Exception e)
        {
          System.out.println(e);
        }
    }
    else
    if(command.compareTo("SQLPRINTCURRENTROW") == 0)
    {
        DebugPrint("/SQLPRINTCURRENTROW");
        StringBuffer tableEntries = new StringBuffer();

        for(int i = 0; i < regD.size(); i++)
        {
            tableEntries.append((String)regD.elementAt(i)+" | ");
        }

        tableEntries.append("\n<br><br>\n");

        return tableEntries.toString(); 
    }
    else
    if(command.compareTo("SQLCLEAR") == 0)
    {
        DebugPrint("/SQLCLEAR");
        regSQL.clear();
    }
    else
    if(command.compareTo("SQLPRINTTABLEDATA") == 0)
    {
       DebugPrint("/SQLPRINTTABLEDATA");
       StringBuffer sqlquerytabledata = new StringBuffer();

       for(int i = 0; i < regSQL.size(); i++)
       {
         Vector entry = (Vector)regSQL.elementAt(i);
         sqlquerytabledata.append("<tr>\n");
         for(int j = 0; j < entry.size(); j++)
         {
           sqlquerytabledata.append("<td>"+(String)entry.elementAt(j)+"</td>\n");
         }
         sqlquerytabledata.append("</tr>\n");
       }

       return sqlquerytabledata.toString(); 
    }
    else
    if(command.compareTo("PRINT") == 0)
    {
      DebugPrint("/PRINT");
      return parseArguments(argument.toString());
    }
    else
    if(command.compareTo("JZ") == 0)
    {
        DebugPrint("/JZ");
        if(flagZ)
        {
            try
               {
                int nNewPointer = Integer.valueOf(argument.toString()).intValue();
                regOpcodePointer = nNewPointer;
               }
               catch(Exception e)
               {
                System.out.println(e);
               }
         }
    }
    else
    if(command.compareTo("JNZ") == 0)
    {
        DebugPrint("/JNZ");
        if(!flagZ)
        {
            StringTokenizer StNewArg = new StringTokenizer(argument.toString(), " ");
            try
               {
                int nNewPointer = Integer.valueOf(StNewArg.nextToken()).intValue();
                regOpcodePointer = nNewPointer;
               }
               catch(Exception e)
               {
                  System.out.println("CaddyVM JNZ "+e);
               }
         }
    }

    return "";//nothing
}

// Parse our "GET" variables
String parseArguments(String string)
{

    StringBuffer buffer = new StringBuffer();

    StringTokenizer ST = new StringTokenizer(string, " =()\",&", true);
    while(ST.hasMoreTokens())
    {
        String token = ST.nextToken();

        if(token.substring(0,1).compareTo("$") == 0)
        {
            if(regHashVars != null)
            {
                buffer.append(regHashVars.get(token));
            }
        }
        else
        {
          buffer.append(token);
        }
   
    }

    return buffer.toString();
}

//Decrement our A register
//Check for Zero condition as well
void DecA()
{
    regA--;
    if(regA < 0)
    {
      regA = 0;
    }    
    
    if(regA == 0)
    {
        flagZ = true;
    }
    else
    {
        flagZ = false;
    }

}

void DebugPrint(String msg)
{
   if(m_bDebug == true)
   System.out.println(msg);
}

}