import java.awt.*;
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;

public class HttpServerCommThread extends Thread
{
DataInputStream m_In;
PrintStream m_Out; 
Socket m_Incoming;
HttpServerThread m_WebServer;
String webroot = "wwwroot/";
String rootpage = "index.ctml";
String m_nServerVersion = "CaddyHTTP Server 1.0";
CaddyVM vm;


public HttpServerCommThread(Socket i, HttpServerThread ws)
{ 
    m_Incoming=i;
    m_WebServer = ws;
    vm = new CaddyVM(m_WebServer.getSQL());
}

public void run()
{
   try
     {
        //Get incomming stream in order to parse GET commands
        m_In = new DataInputStream(m_Incoming.getInputStream());
        String sIncommingText = m_In.readLine();

        //Clear the incomming stream
	String str = m_In.readLine();
	//System.out.println("in: "+str);
        while(str.length() != 0)
	{
		str = m_In.readLine();
		//System.out.println("in: "+str);
	}
	
        //System.out.println("Http: IN: "+sIncommingText);
        m_Out = new PrintStream(m_Incoming.getOutputStream());
      
        //Return response
        m_Out.print("HTTP/1.1 200 OK\n");
        m_Out.print("Server: "+m_nServerVersion+"\n");
 	
	//Handle the incomming stream
	handleInput(sIncommingText, m_Incoming.getOutputStream());		
		
        m_Incoming.close();
		System.out.println("  Closed Connection...");
       }
    catch (Exception e)
         {
            System.out.println(e);
            System.out.println("Error (HttpServerCommThread) :"+e);
         }     
}  
  
String handleInput(String sInput, OutputStream m_Out)
{

	if(!ValidateInput(sInput))
	{
		PrintStream printStream = new PrintStream(m_Out);
		printStream.print("Content-Type: text/html\n\n");
		printStream.print(getErrorMessage(404) + "\n\n");
		System.out.println("Error! Failed Auth.. connection tried to use ../");
		return "Error! Failed Auth";
	}
	
	//System.out.println("in: "+sInput);
	StringTokenizer ST = new StringTokenizer(sInput, " ");
	Hashtable argumentTable = new Hashtable();

    while(ST.hasMoreTokens())
    {
        String token = ST.nextToken();
        if(token.compareTo("GET") == 0)
        {
            if(ST.hasMoreTokens())
            {
                StringTokenizer webArguments = new StringTokenizer(ST.nextToken(), "?");
                String page = webArguments.nextToken();
                String arguments = "";
                if(webArguments.hasMoreTokens())
                {
                   arguments = webArguments.nextToken();
                }
                hashArguments(arguments, argumentTable);
                System.out.print("  page = "+page+"\n");
		//System.out.println("args = "+arguments);

                if(page.compareTo("/") == 0)
				{
					String extension = GetPageType(rootpage);
					
					// For now the root page MUST be an CTML file
					if(extension.compareTo("ctml") == 0)
					{
					  new CTMLHandler(m_Out, vm, argumentTable, getPage(rootpage));
					}
				}
                else
				{	
					// Handle the request by filetype
					String extension = GetPageType(page);
					if(extension.compareTo("jpg") == 0)
					{
						new ImageHandler(m_Out, "image/jpeg", webroot+page);
					}else
					if(extension.compareTo("gif") == 0)
					{
						new ImageHandler(m_Out, "image/gif", webroot+page);
					}else
					if(extension.compareTo("png") == 0)
					{
						new ImageHandler(m_Out, "image/png", webroot+page);
					}else					
					if(
					(extension.compareTo("ctml") == 0)||
					(extension.compareTo("html") == 0)||
					(extension.compareTo("htm") == 0)
				        )
					{
					  new CTMLHandler(m_Out, vm, argumentTable, getPage(page));
					}else
					{
					  //unknown extention.. must be binary
	                                  new DataHandler(m_Out, "binary/octet-stream", webroot+page);						

					}
				}                            
            }
            else
            {
              //TODO: send defalt error page
              return "Error, Blank GET command";
            }
        }

    }
return "Nothing\n\n";
}

void hashArguments(String arguments, Hashtable hashtable)
{

    StringTokenizer ST = new StringTokenizer(arguments, "&");
    while(ST.hasMoreTokens())
    {
        StringTokenizer stArg = new StringTokenizer(ST.nextToken(), "=");

        String key = "";
        if(stArg.hasMoreTokens())
        {
            key = stArg.nextToken();
        }        
        
        String value = "";
        if(stArg.hasMoreTokens())
        {
            value = stArg.nextToken();
        }

        if((key.length() > 0) && (value.length() > 0))
        {
            //Add an entry in the hash table
            hashtable.put("$"+key, value);        
        }
    
    }
}

String GetPageType(String page)
{
	StringTokenizer ST = new StringTokenizer(page, ".");
	String extension = ST.nextToken();
	while(ST.hasMoreTokens())
	{
		extension = ST.nextToken();
	}
	//System.out.println("extension: "+extension);
	
	return extension;
}


String getPage(String sPageName)
{

	String text = "";
	
	try	
    {
		byte[] data;
		File F = new File(webroot+sPageName);
		FileInputStream fin = new FileInputStream(F);
		int filesize = (int)F.length();
		data = new byte[filesize];
		fin.read (data, 0, filesize);
		fin.close();
		text = new String (data);
	}
    catch (Exception e)
    {
        return getErrorMessage(404);//System.out.println("File Not Found: "+e);
    }

return text;
}

String getErrorMessage(int code)
{

    switch(code)
    {
        case 404:
        {
            return "<font size=+3>404 - File Not Found!!!</font>";    
        }
        default:
        {
            return "<font size=+3>404 - File Not Found!!!</font>";
        }
     }

    //return "<font size=+3>404 - File Not Found!!!</font>";
}

boolean ValidateInput(String input)
{

	StringBuffer SB = new StringBuffer(input);
	for(int i = 0; i < SB.length(); i++)
	{
		if( ( SB.charAt(i) == '.' ) && ( i != SB.length() ) )
		{
			if( SB.charAt(i+1) == '.' )
			{
				return false;
			}
		}
	}
	return true;
}

} // end all


